import {
	CanActivate,
	ExecutionContext,
	Inject,
	Injectable,
	UnauthorizedException,
} from "@nestjs/common";
import { Reflector } from "@nestjs/core";
import { CredentialsService } from "apps/backend/src/app/credentials/credentials.service";
import { DbService } from "apps/backend/src/app/db/db.service";
import { UsersTable } from "apps/backend/src/app/db/schema";
import { eq } from "drizzle-orm";
import type { Request } from "express";

@Injectable()
export class UserGuard implements CanActivate {
	constructor(
		@Inject(CredentialsService)
		private readonly credentialService: CredentialsService,
		@Inject(DbService) private readonly databaseService: DbService,
	) {}

	async canActivate(context: ExecutionContext): Promise<boolean> {
		const request: Request = context.switchToHttp().getRequest();
		const authHeader = request.headers.authorization;

		if (!authHeader)
			throw new UnauthorizedException("No authorization header found.");

		const token = authHeader.split(" ")[1];
		const vToken = await this.credentialService.verifyAuthToken(token);

		const user = await this.databaseService
			.use()
			.select()
			.from(UsersTable)
			.where(eq(UsersTable.uuid, vToken.payload.sub));

		if (user.length !== 1)
			throw new UnauthorizedException("No such user found.");
		/*
    if (user[0].emailCode)
      throw new UnauthorizedException("Email not verified.");

   */

		// Inject user ID into request body
		request.body.sourceUserId = vToken.payload.sub;

		return true;
	}
}

@Injectable()
export class AdminGuard implements CanActivate {
	constructor(
		@Inject(CredentialsService)
		private readonly credentialService: CredentialsService,
		@Inject(DbService) private readonly databaseService: DbService,
	) {}
	async canActivate(context: ExecutionContext): Promise<boolean> {
		const request: Request = context.switchToHttp().getRequest();

		const authHeader = request.headers.authorization;
		if (!authHeader) {
			throw new UnauthorizedException("No authorization header found.");
		}
		const token = authHeader.split(" ")[1];
		const vToken = await this.credentialService.verifyAuthToken(token);

		const user = await this.databaseService
			.use()
			.select()
			.from(UsersTable)
			.where(eq(UsersTable.uuid, vToken.payload.sub));

		if (user.length !== 1)
			throw new UnauthorizedException("No such user found.");

		if (!user[0].isAdmin) {
			throw new UnauthorizedException("Administrator only..");
		}

		// Inject user ID into request body
		request.body.sourceUserId = vToken.payload.sub;

		return true;
	}
}
